package main

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/google/uuid"
	"github.com/olivere/elastic/v7"
	"gopkg.in/yaml.v3"
	"gorm.io/driver/mysql"
	"gorm.io/driver/postgres"
	"gorm.io/gorm"
	"log"
	"os"
	"strconv"
	"strings"
	"time"
)

// Config represents the structure of the configuration file
type Config struct {
	Postgres      PostgresConfig      `json:"postgres" yaml:"postgres"`
	MySQL         MySQLConfig         `json:"mysql" yaml:"mysql"`
	Elasticsearch ElasticsearchConfig `json:"elasticsearch" yaml:"elasticsearch"`
	Query         QueryConfig         `json:"query" yaml:"query"`
}

type MySQLConfig struct {
	Host     string `json:"host" yaml:"host"`
	User     string `json:"user" yaml:"user"`
	Password string `json:"password" yaml:"password"`
	DBName   string `json:"dbname" yaml:"dbname"`
	Port     string `json:"port" yaml:"port"`
	Charset  string `json:"charset" yaml:"charset"`
}

type PostgresConfig struct {
	Host       string `json:"host" yaml:"host"`
	User       string `json:"user" yaml:"user"`
	Password   string `json:"password" yaml:"password"`
	DBName     string `json:"dbname" yaml:"dbname"`
	Port       string `json:"port" yaml:"port"`
	SSLMode    string `json:"sslmode" yaml:"sslmode"`
	TimeZone   string `json:"timezone" yaml:"timezone"`
	Charset    string `json:"charset" yaml:"charset"`
	SchemaName string `json:"schemaname" yaml:"schemaname"`
}

type ElasticsearchConfig struct {
	URL   string `json:"url" yaml:"url"`
	Index string `json:"index" yaml:"index"`
}

type QueryConfig struct {
	StartTime string `json:"start_time" yaml:"start_time"`
	EndTime   string `json:"end_time" yaml:"end_time"`
}

// HistoryLog represents the structure of the history_log table
type HistoryLog struct {
	Id               string    `gorm:"primaryKey;type:varchar(32);comment:主键，来源于 jk_history_business_service_api 表的 id 字段" json:"id"`                       // 主键
	IdSysLog         string    `gorm:"type:varchar(32);comment:来源于 sys_log 表的 id 字段，用于保留 sys_log 表中的 id 值" json:"id_sys_log"`                               // 来源于 sys_log 表的 id
	LogType          int       `gorm:"type:int;comment:日志类型（1登录日志，2操作日志），来源于 sys_log 表的 log_type 字段" json:"log_type"`                                       // 日志类型
	LogContent       string    `gorm:"type:varchar(1000);comment:日志内容，来源于 sys_log 表的 log_content 字段" json:"log_content"`                                    // 日志内容
	OperateType      int       `gorm:"type:int;comment:操作类型，来源于 sys_log 表的 operate_type 字段" json:"operate_type"`                                            // 操作类型
	Userid           string    `gorm:"type:varchar(32);comment:操作用户账号，来源于 sys_log 表的 userid 字段" json:"userid"`                                              // 操作用户账号
	Username         string    `gorm:"type:varchar(100);comment:操作用户名称，来源于 sys_log 表的 username 字段" json:"username"`                                         // 操作用户名称
	RemoteIp         string    `gorm:"type:varchar(100);comment:IP地址，来源于 sys_log 表的 ip 字段" json:"remote_ip"`                                                // IP地址
	Method           string    `gorm:"type:varchar(500);comment:请求的 Java 方法，来源于 sys_log 表的 method 字段" json:"method"`                                        // 请求的 Java 方法
	RequestUrl       string    `gorm:"type:varchar(255);comment:请求路径，来源于 sys_log 表的 request_url 字段" json:"request_url"`                                     // 请求路径
	InParams         string    `gorm:"type:text;comment:请求参数，来源于 sys_log 表的 request_param 字段" json:"in_params"`                                             // 请求参数
	RequestType      string    `gorm:"type:varchar(10);comment:请求类型，来源于 sys_log 表的 request_type 字段" json:"request_type"`                                    // 请求类型
	Duration         int64     `gorm:"type:int;comment:耗时，来源于 sys_log 表的 cost_time 字段" json:"cost_time"`                                                    // 耗时
	CreateBySyslog   string    `gorm:"type:varchar(32);comment:创建人，来源于 sys_log 表的 create_by 字段" json:"create_by_syslog"`                                    // 创建人
	CreateTimeSyslog time.Time `gorm:"type:timestamp;comment:创建时间，来源于 sys_log 表的 create_time 字段" json:"create_time_syslog"`                                 // 创建时间
	UpdateBySyslog   string    `gorm:"type:varchar(32);comment:更新人，来源于 sys_log 表的 update_by 字段" json:"update_by_syslog"`                                    // 更新人
	UpdateTimeSyslog time.Time `gorm:"type:timestamp;comment:更新时间，来源于 sys_log 表的 update_time 字段" json:"update_time_syslog"`                                 // 更新时间
	OutParams        string    `gorm:"type:text;comment:响应参数，来源于 sys_log 表的 resp_param 字段" json:"out_params"`                                               // 响应参数
	HistoryId        string    `gorm:"type:varchar(100);comment:关联的 jk_history_business_service_api 表主键 id，来源于 sys_log 表的 history_id 字段" json:"history_id"` // 关联的 jk_history_business_service_api 表主键 id

	// 从 jk_history_business_service_api 表继承的字段
	ServiceId          string    `gorm:"type:varchar(64);not null;default:'';comment:场景服务ID，来源于 jk_history_business_service_api 表的 scene_service_id 字段" json:"service_id"`          // 场景服务ID
	Url                string    `gorm:"type:varchar(250);comment:节点ID，来源于 jk_history_business_service_api 表的 node_id 字段" json:"url"`                                               // 节点ID
	ServiceCode        string    `gorm:"type:varchar(64);comment:业务服务CODE，来源于 jk_history_business_service_api 表的 business_service_api_code 字段" json:"service_code"`                 // 业务服务CODE
	State              string    `gorm:"type:varchar(20);not null;default:'';comment:调用状态，来源于 jk_history_business_service_api 表的 status 字段" json:"state"`                           // 调用状态
	AccessUuid         string    `gorm:"type:varchar(70);comment:应用APP key，来源于 jk_history_business_service_api 表的 app_key 字段" json:"access_uuid"`                                   // 应用APP key
	RegionCode         string    `gorm:"type:varchar(45);not null;default:'';comment:区域CODE(省或市或区)，来源于 jk_history_business_service_api 表的 region_code 字段" json:"region_code"`       // 区域CODE(省或市或区)
	RegionCodeCity     string    `gorm:"type:varchar(45);not null;default:'';comment:区域CODE(市)，来源于 jk_history_business_service_api 表的 region_code_city 字段" json:"region_code_city"` // 区域CODE(市)
	Kind               string    `gorm:"type:varchar(20);not null;default:'';comment:类型，来源于 jk_history_business_service_api 表的 kind 字段" json:"kind"`                                // 类型
	Uuid               string    `gorm:"type:varchar(500);comment:唯一标识符，来源于 jk_history_business_service_api 表的 uuid 字段" json:"uuid"`                                                // 唯一标识符
	Cost               int       `gorm:"type:int;comment:耗时，来源于 jk_history_business_service_api 表的 cost 字段" json:"duration"`                                                        // 耗时
	RecordDay          time.Time `gorm:"type:date;comment:记录日期，来源于 jk_history_business_service_api 表的 record_day 字段" json:"record_day"`                                             // 记录日期
	ThemeType          string    `gorm:"type:varchar(45);not null;default:'';comment:场景服务类型，来源于 jk_history_business_service_api 表的 service_type 字段" json:"theme_type"`              // 场景服务类型
	AccessXzqhdm       string    `gorm:"type:varchar(45);comment:外部传递的区县code，来源于 jk_history_business_service_api 表的 area_code 字段" json:"access_xzqhdm"`                             // 外部传递的区县code
	GroupId            string    `gorm:"type:varchar(64);comment:分组ID，来源于 jk_history_business_service_api 表的 group_id 字段" json:"group_id"`                                          // 分组ID
	ResponseStatusCode int       `gorm:"type:int;comment:接口响应状态码，来源于 jk_history_business_service_api 表的 status_code 字段" json:"response_status_code"`                                // 接口响应状态码
}

// ESLog represents the Elasticsearch log structure
type ESLog struct {
	ID              string                 `json:"id"`
	TenantID        string                 `json:"tenant_id"`
	Name            string                 `json:"name"`
	Method          string                 `json:"method"`
	ParentID        string                 `json:"parent_id"`
	TraceID         string                 `json:"trace_id"`
	Event           string                 `json:"event"`
	Duration        int                    `json:"duration"`
	TimestampMillis string                 `json:"timestamp_millis"`
	Timestamp       int64                  `json:"timestamp"`
	Tags            map[string]interface{} `json:"tags"`
	Q               []string               `json:"_q"`
}

// LoadConfig loads the configuration from a file
func LoadConfig(filename string) (*Config, error) {
	data, err := os.ReadFile(filename)
	if err != nil {
		return nil, err
	}

	var config Config
	if ext := GetFileExtension(filename); ext == ".json" {
		err = json.Unmarshal(data, &config)
	} else if ext == ".yaml" || ext == ".yml" {
		err = yaml.Unmarshal(data, &config)
	}

	return &config, err
}

// GetFileExtension returns the file extension
func GetFileExtension(filename string) string {
	return filename[len(filename)-5:]
}

func main() {
	// 加载配置文件
	configFile := "config.yaml"
	config, err := LoadConfig(configFile)
	if err != nil {
		log.Fatalf("failed to load config: %v", err)
	}

	// 配置 PostgreSQL 连接信息
	postgres_dsn := fmt.Sprintf("host=%s user=%s password=%s dbname=%s port=%s sslmode=%s TimeZone=%s client_encoding=%s",
		config.Postgres.Host, config.Postgres.User, config.Postgres.Password, config.Postgres.DBName, config.Postgres.Port, config.Postgres.SSLMode, config.Postgres.TimeZone, config.Postgres.Charset)
	postgres_db, err := gorm.Open(postgres.New(postgres.Config{
		DSN:                  postgres_dsn,
		PreferSimpleProtocol: false, // disables implicit prepared statement usage
	}), &gorm.Config{})
	if err != nil {
		log.Fatalf("failed to connect database: %v", err)
	}

	// 获取服务相关信息
	serviceMap, err := GetServiceThemeInfo(config.Postgres.SchemaName, postgres_db)
	if err != nil {
		log.Fatalf("failed to get service theme info: %v", err)
	}

	// 获取系统名称
	systemMap, err := GetServiceAccessInfo(postgres_db)
	if err != nil {
		log.Fatalf("failed to get service access builder info: %v", err)
	}

	// 配置 MySQL 连接信息
	mysql_dsn := fmt.Sprintf("%s:%s@tcp(%s:%s)/%s?charset=%s&parseTime=True&loc=Local",
		config.MySQL.User, config.MySQL.Password, config.MySQL.Host, config.MySQL.Port, config.MySQL.DBName, config.MySQL.Charset)
	mysql_db, err := gorm.Open(mysql.Open(mysql_dsn), &gorm.Config{})
	if err != nil {
		log.Fatalf("failed to connect to MySQL: %v", err)
	}

	// 查询时间配置
	startTime := config.Query.StartTime
	endTime := config.Query.EndTime

	// 查询 MySQL 中的数据
	var logs []HistoryLog
	if err := mysql_db.Model(HistoryLog{}).Table("history_log_view").
		Where("create_time_syslog BETWEEN ? AND ?", startTime, endTime).Find(&logs).Error; err != nil {
		log.Fatalf("failed to query logs: %v", err)
	}

	// Elasticsearch 连接
	esClient, err := elastic.NewClient(elastic.SetURL(config.Elasticsearch.URL), elastic.SetSniff(false))
	if err != nil {
		log.Fatalf("failed to create Elasticsearch client: %v", err)
	}

	// 统计写入和失败的数量
	totalCount := len(logs)
	successCount := 0
	failedIDs := []string{}

	// 处理每条日志并写入 Elasticsearch
	for _, logItem := range logs {
		uuidStr := uuid.New().String()
		fmt.Println(uuidStr)
		instanceId := strings.ReplaceAll(uuid.New().String(), "-", "")
		// 构造 Elasticsearch 日志结构
		esLog := ESLog{
			ID:              uuidStr,                                                // 日志主键
			TenantID:        "<nil>",                                                // 租户ID
			Name:            "0.1.0",                                                // 固定版本号
			Event:           "scp-default",                                          // 事件
			Method:          "ProxyService.Redirect",                                // 请求方法
			ParentID:        "no-biz-id",                                            // 父ID
			TraceID:         "no-trace-id",                                          // 追踪ID
			Duration:        int(logItem.Duration),                                  // 请求耗时
			TimestampMillis: logItem.CreateTimeSyslog.Format("2006-01-02 15:04:05"), // 创建时间的字符串格式
			Timestamp:       logItem.CreateTimeSyslog.UnixMilli(),                   // 创建时间的Unix毫秒时间戳
			Tags: map[string]interface{}{
				"id":                   logItem.Id,                                               // 日志主键
				"access_uuid":          logItem.AccessUuid,                                       // 应用UUID
				"access_xzqhdm":        logItem.AccessXzqhdm,                                     // 外部传递的区县code
				"duration":             strconv.FormatInt(logItem.Duration, 10),                  // 请求耗时
				"region_code":          logItem.RegionCode,                                       // 区域代码
				"service_code":         logItem.ServiceCode,                                      // 业务服务代码
				"state":                logItem.State,                                            // 调用状态
				"logClassify":          "service",                                                // 业务类型
				"service_id":           logItem.ServiceId,                                        // 场景服务ID
				"url":                  logItem.RequestUrl,                                       // 节点URL
				"theme_type":           logItem.ThemeType,                                        // 场景服务类型
				"response_status_code": strconv.FormatInt(int64(logItem.ResponseStatusCode), 10), // 接口响应状态码
				"remote_ip":            logItem.RemoteIp,                                         // 请求IP地址
				"request_url":          logItem.RequestUrl,                                       // 请求路径
				"msg":                  logItem.LogContent,                                       // 日志内容
				"in_params":            logItem.InParams,                                         // 请求参数
				"out_params":           logItem.OutParams,                                        // 响应参数
				"in_time":              logItem.CreateTimeSyslog.Format("2006-01-02 15:04:05"),   // 创建时间
				"old_log":              "sm_history_log",                                         // 历史日志标识
				"scpAppName":           "scp-service-manager",                                    // app 标识
				"link_no":              "0",                                                      //
				"InitiatorLogKey":      "需求方",                                                    // 发起方
				"ReceiverLogKey":       "服务管理",                                                   // 接收方
				"instance_id":          instanceId,                                               // 实例id
			},
			Q: []string{
				fmt.Sprintf("id=%s", logItem.Id),                                                                 // 日志主键
				fmt.Sprintf("access_uuid=%s", logItem.AccessUuid),                                                // 应用UUID
				fmt.Sprintf("duration=%s", strconv.FormatInt(logItem.Duration, 10)),                              // 请求耗时
				fmt.Sprintf("region_code=%s", logItem.RegionCode),                                                // 区域代码
				fmt.Sprintf("service_code=%s", logItem.ServiceCode),                                              // 业务服务代码
				fmt.Sprintf("state=%s", logItem.State),                                                           // 调用状态
				fmt.Sprintf("logClassify=service"),                                                               // 业务类型
				fmt.Sprintf("service_id=%s", logItem.ServiceId),                                                  // 场景服务ID
				fmt.Sprintf("url=%s", logItem.Url),                                                               // 节点URL
				fmt.Sprintf("theme_type=%s", logItem.ThemeType),                                                  // 场景服务类型
				fmt.Sprintf("response_status_code=%s", strconv.FormatInt(int64(logItem.ResponseStatusCode), 10)), // 接口响应状态码
				fmt.Sprintf("access_xzqhdm=%s", logItem.AccessXzqhdm),                                            // 外部传递的区县code
				fmt.Sprintf("remote_ip=%s", logItem.RemoteIp),                                                    // 请求IP地址
				fmt.Sprintf("request_url=%s", logItem.RequestUrl),                                                // 请求路径
				fmt.Sprintf("in_params=%s", logItem.InParams),                                                    // 请求参数
				fmt.Sprintf("out_params=%s", logItem.OutParams),                                                  // 响应参数
				fmt.Sprintf("in_time=%s", logItem.CreateTimeSyslog.Format("2006-01-02 15:04:05")),                // 创建时间
				fmt.Sprintf("old_log=sm_history_log"),                                                            // 历史日志标识
				fmt.Sprintf("scpAppName=scp-service-manager"),                                                    // app 标识
				fmt.Sprintf("link_no=0"),                                                                         //
				fmt.Sprintf("InitiatorLogKey=需求方"),                                                               // 发起方
				fmt.Sprintf("ReceiverLogKey=服务管理"),                                                               // 接收方
				fmt.Sprintf("instance_id=%s", instanceId),                                                        // 实例id
			},
		}

		if len(logItem.LogContent) < 32000 {
			esLog.Q = append(esLog.Q, fmt.Sprintf("msg=%s", logItem.LogContent))
		}

		if serviceInfo, ok := serviceMap[logItem.ServiceId]; ok {
			esLog.Tags["service_name"] = serviceInfo.ServiceName
			esLog.Tags["theme_name"] = serviceInfo.ThemeName
			esLog.Tags["theme_code"] = serviceInfo.ThemeCode
			esLog.Q = append(esLog.Q, fmt.Sprintf("service_name=%s", serviceInfo.ServiceName))
			esLog.Q = append(esLog.Q, fmt.Sprintf("theme_name=%s", serviceInfo.ThemeName))
			esLog.Q = append(esLog.Q, fmt.Sprintf("theme_code=%s", serviceInfo.ThemeCode))
		}

		if systemName, ok := systemMap[logItem.AccessUuid]; ok {
			esLog.Tags["access_system_name"] = systemName
			esLog.Q = append(esLog.Q, fmt.Sprintf("access_system_name=%s", systemName))
		}

		indexStr := config.Elasticsearch.Index + logItem.CreateTimeSyslog.Format("2006-01-02")
		fmt.Println(indexStr)
		// 写入 Elasticsearch
		_, err := esClient.Index().
			Index(indexStr).
			Id(esLog.ID).
			BodyJson(esLog).
			Do(context.Background())
		if err != nil {
			failedIDs = append(failedIDs, logItem.Id)
		} else {
			successCount++
		}
	}

	// 输出结果
	fmt.Printf("Total Logs: %d, Success: %d, Failed: %d\n", totalCount, successCount, len(failedIDs))
	if len(failedIDs) > 0 {
		fmt.Printf("Failed IDs: %v\n", failedIDs)
	}
}
