package main

import (
	"gorm.io/gorm"
)

type Service struct {
	Id          string `gorm:"primaryKey;type:char(32)" json:"id"`
	ServiceName string `gorm:"type:varchar(255);comment:服务名称" json:"service_name"`
	Url         string `gorm:"type:varchar(255);comment:服务对外地址" json:"url"`
	DelFlag     int8   `gorm:"comment:是否删除 0：正常 1：已删除" json:"del_flag"`
}

type RelationshipsThemeService struct {
	Id        string `gorm:"primaryKey;type:char(32)" json:"id"`
	ServiceId string `gorm:"type:char(32);comment:服务 id" json:"service_id"`
	ThemeId   string `gorm:"type:char(32);comment:服务主题 id" json:"theme_id"`
	DelFlag   int8   `gorm:"comment:是否删除 0：正常 1：已删除" json:"del_flag"`
}

type ServiceTheme struct {
	Id        string `gorm:"primaryKey;type:char(32)" json:"id"`
	ThemeCode string `gorm:"type:varchar(255);comment:服务主题编码" json:"theme_code"`
	ThemeName string `gorm:"type:varchar(45);comment:服务主题名称" json:"theme_name"`
	DelFlag   int8   `gorm:"comment:是否删除 0：正常 1：已删除" json:"del_flag"`
}

type ServiceResult struct {
	Id          string `json:"id"`
	ServiceName string `json:"service_name"`
	Url         string `json:"url"`
	ThemeCode   string `json:"theme_code"`
	ThemeName   string `json:"theme_name"`
}

func GetServiceThemeInfo(schemaName string, db *gorm.DB) (map[string]ServiceResult, error) {
	var results []ServiceResult
	serviceMap := make(map[string]ServiceResult)

	// 动态生成表名
	serviceInfoTable := schemaName + ".service_info"
	relationshipsTable := schemaName + ".relationships_theme_service"
	themeTable := schemaName + ".theme_service"

	// 查询所有符合条件的 service_info, relationships_theme_service, 和 service_theme 表
	err := db.Table(serviceInfoTable).
		Select("service_info.id, service_info.service_name, service_info.url, theme_service.theme_code, theme_service.theme_name").
		Joins("JOIN " + relationshipsTable + " ON service_info.id = " + relationshipsTable + ".service_id").
		Joins("JOIN " + themeTable + " ON " + relationshipsTable + ".theme_id = " + themeTable + ".id").
		Where("service_info.del_flag = 0 AND " + relationshipsTable + ".del_flag = 0 AND " + themeTable + ".del_flag = 0").
		Scan(&results).Error

	if err != nil {
		return nil, err
	}

	// 将结果按 service_id 存入 map
	for _, result := range results {
		serviceMap[result.Id] = result
	}

	return serviceMap, nil
}

// ServiceAccess 服务接入信息
type ServiceAccess struct {
	Id         string `gorm:"primaryKey;type:char(32)" json:"id"`                   // 主键id
	BuilderId  string `gorm:"type:char(32);comment:建设单位id 需求方id" json:"builder_id"` // 建设单位id 需求方id
	AccessUuid string `gorm:"type:varchar(70);comment:接入唯一码" json:"access_uuid"`    // 接入唯一码
	DelFlag    int8   `gorm:"comment:是否删除 0：正常 1：已删除" json:"del_flag"`              // 是否删除 0：正常 1：已删除
}

// AccessBuilder 建设单位信息
type AccessBuilder struct {
	Id         string `gorm:"primaryKey;type:char(32)" json:"id"`               // 主键id
	SystemName string `gorm:"type:varchar(45);comment:系统名称" json:"system_name"` // 系统名称
	DelFlag    int8   `gorm:"comment:是否删除 0：正常 1：已删除" json:"del_flag"`          // 是否删除 0：正常 1：已删除
}

// GetServiceAccessInfo 查询服务接入信息和建设单位信息
func GetServiceAccessInfo(db *gorm.DB) (map[string]string, error) {
	var results []struct {
		AccessUuid string
		SystemName string
	}

	// 执行查询，关联 service_access 和 access_builder 表，并且保证各个表的 del_flag = 0
	err := db.Table("service_access").
		Select("service_access.access_uuid, access_builder.system_name").
		Joins("JOIN access_builder ON service_access.builder_id = access_builder.id").
		Where("service_access.del_flag = 0 AND access_builder.del_flag = 0").
		Scan(&results).Error

	if err != nil {
		return nil, err
	}

	// 将查询结果存入 map[string]string
	resultMap := make(map[string]string)
	for _, result := range results {
		resultMap[result.AccessUuid] = result.SystemName
	}

	return resultMap, nil
}
